# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import GameState
import GridUtils
import random

def flip_lights(grid, pos):
    """" flips 1 to 0 and 0 to 1 for pos and each orthogonal neighbor """
    grid[pos] = 1 - grid[pos]
    for np,nval in GridUtils.enumerate_ortho_neighbors(grid, pos):
        grid[np] = 1 - nval

class LightsOutGameState(GameState):
    def __init__(self, dimensions=None, params=None, players=2, reset=True):
        GameState.__init__(self, dimensions, params, 1, reset)
        if reset:
            for pos in self.itercoords():
                if random.randint(0,1)==1:
                    flip_lights(self.grid, pos)
                    
    def successors(self):
        for pos in self.itercoords():
            nextgrid = self.grid.copy()
            flip_lights(nextgrid, pos)
            yield ( (pos,), self.next_state_with_grid(nextgrid) )

    def has_scores(self):
        return False
            
    def is_game_over(self):
        return (1 not in self.grid)
        
    # default animation is correct for adding new and removing existing pieces
    